#include <string>
#include <map>
#ifndef _WIN32
#include <syslog.h>
#endif
#include "tinyxml.h"
#include <log4cplus/logger.h>


bool load_config(const std::string & filename,  std::string & endpoint, std::string & api_key, std::string & feed, std::map<std::string, int> & datasets, int & update_frequency, int & min_values, std::string & msg_format)
{
	log4cplus::Logger logger=log4cplus::Logger::getRoot();
	TiXmlDocument doc(filename.c_str());
	doc.LoadFile();
	std::ostringstream log_message;
	log_message << "invalid config file "<< filename << "(" << doc.ErrorDesc() << ")";
	if(doc.Error()) {	
		LOG4CPLUS_ERROR(logger, log_message);
		return false;	
	}
	TiXmlNode *root = doc.FirstChild("config");
	if(!root) {
		LOG4CPLUS_ERROR(logger, "invalid config file (no root element)");
		return false;	
	}

	TiXmlNode *ep = root->FirstChild("endpoint");
	if(ep) {
		endpoint = ep->FirstChild()->Value();
	}
	else {
		LOG4CPLUS_ERROR(logger, "endpoint not specified in config");
		return false;
	}

	TiXmlNode *ky = root->FirstChild("api_key");
	if(ky) {
		api_key = ky->FirstChild()->Value();
	}
	else {
		LOG4CPLUS_ERROR(logger, "api key not specified in config");
		return false;
	}

	TiXmlNode *fq = root->FirstChild("update_frequecy");
	if(fq) {
		update_frequency = atol(fq->FirstChild()->Value());
	}

	TiXmlNode *mv = root->FirstChild("min_values");
	if(mv) {
		min_values = atol(mv->FirstChild()->Value());
	}

	TiXmlNode *fd = root->FirstChild("feed");
	if(fd) {
		feed = fd->FirstChild()->Value();
	}
	else {
		LOG4CPLUS_ERROR(logger, "feed not specified in config");
		return false;
	}

	TiXmlNode *sensors = root->FirstChild("sensors");
	TiXmlNode *sensor = sensors->FirstChild("sensor");
	while(sensor) {
		TiXmlNode *key = sensor->FirstChild("key");
		TiXmlNode *dataset = sensor->FirstChild("dataset");
		datasets[key->FirstChild()->Value()] = atol(dataset->FirstChild()->Value());
		sensor=sensor->NextSibling("sensor");
	}

	if(datasets.size()==0)
	{
		LOG4CPLUS_ERROR(logger, "dataset not specified in config");
		return false;
	}
	
	TiXmlNode *msg_format_node = root->FirstChild("msg_format");
	if(msg_format_node) {
		msg_format = msg_format_node->FirstChild()->Value();
	}

	return true;
}

